Module arrow

Module arrow 

Composable function wrappers with Category and Strong instances.

The Arrow trait provides composable, callable wrappers over closures. It extends Category and Strong, aligning with Haskell’s Arrow type class.

Examples

use fp_library::{
	brands::*,
	functions::*,
};

let f = arrow::<RcFnBrand, _, _>(|x: i32| x * 2);
assert_eq!(f(5), 10);

Traits

Arrow
A trait for composable function wrappers with Category and Strong instances.