map

Function map 

pub fn map<'a, Brand: Functor, A: 'a, B: 'a>(
    f: impl Fn(A) -> B + 'a,
    fa: <Brand as Kind_cdc7cd43dac7585f>::Of<'a, A>,
) -> <Brand as Kind_cdc7cd43dac7585f>::Of<'a, B>
Maps a function over the values in the functor context.

Free function version that dispatches to the type class’ associated function.

Type Signature

forall Brand A B. Functor Brand => (A -> B, Brand A) -> Brand B

Type Parameters

  • 'a: The lifetime of the values.
  • Brand: The brand of the functor.
  • A: The type of the value(s) inside the functor.
  • B: The type of the result(s) of applying the function.

Parameters

  • f: The function to apply to the value(s) inside the functor.
  • fa: The functor instance containing the value(s).

Returns

A new functor instance containing the result(s) of applying the function.

Examples

use fp_library::{
	brands::*,
	functions::*,
};

let x = Some(5);
let y = map::<OptionBrand, _, _>(|i| i * 2, x);
assert_eq!(y, Some(10));