The identity element for Alt, forming a monoid on type constructors.
Plus is to type constructors as Monoid is to concrete types.
Examples
use fp_library::{
brands::*,
classes::*,
functions::{
explicit::alt,
*,
},
};
let x = alt::<OptionBrand, _, _, _>(plus_empty::<OptionBrand, i32>(), Some(5));
assert_eq!(x, Some(5));