Module explicit

Module explicit 

Explicit dispatch functions requiring a Brand turbofish.

For most use cases, prefer the inference-enabled wrappers from functions.

Functions

wilt
Partitions a structure based on a function returning a Result in an applicative context.
wither
Maps a function over a data structure and filters out None results in an applicative context.