Module docs

Module docs 

Design documentation for fp-library.

Each submodule contains one design document from the docs/ directory, with cross-document links rewritten as rustdoc intra-doc links. This makes links between documents work in rendered documentation (docs.rs and local cargo doc builds).

Modules

architecture
Architecture & Design Decisions
benchmarking
Benchmarking Comparison
brand_inference
Brand Inference
coyoneda
Coyoneda Implementations
dispatch
Val/Ref Dispatch
features
Features
hkt
Higher-Kinded Types (HKT)
impl_trait_vs_named_generics
impl Trait vs Named Generics
lazy_evaluation
Lazy Evaluation & Effect System
lifetime_ablation_experiment
Lifetime Ablation Experiment
limitations_and_workarounds
Limitations and Workarounds
optics_analysis
Optics Comparison: Rust fp-library vs PureScript purescript-profunctor-lenses
parallelism
Thread Safety and Parallelism
pointer_abstraction
Pointer Abstraction & Shared Semantics
profunctor_analysis
Profunctor Classes: Rust vs PureScript Analysis
project_structure
Project Structure
release_process
Release Process
std_coverage_checklist
Checklist: fp-library vs Rust’s std library
zero_cost
Zero-Cost Abstractions & Uncurried Semantics