Module explicit

Module explicit 

Explicit dispatch functions requiring a Brand turbofish.

For most use cases, prefer the inference-enabled wrappers from the parent functions module.

Re-exports

pub use crate::classes::semiapplicative::apply;

Functions

alt
Combines two values in a context, choosing associatively.
apply_first
Sequences two applicative actions, keeping the result of the first.
apply_second
Sequences two applicative actions, keeping the result of the second.
bi_fold_left
Folds a bifoldable structure from the left.
bi_fold_map
Maps elements of both types to a monoid and combines them.
bi_fold_right
Folds a bifoldable structure from the right.
bi_traverse
Traverses a bitraversable structure, mapping each element to a computation and combining the results.
bimap
Maps two functions over the values in a bifunctor context.
bind
Sequences a monadic computation with a function that produces the next computation.
bind_flipped
Binds a monadic value to a function (flipped argument order).
compact
Removes None values from a container of Options, unwrapping the Some values.
contramap
Contravariantly maps a function over a contravariant functor.
filter
Filters the values in a filterable context using a predicate.
filter_map
Filters and maps the values in a filterable context.
filter_map_with_index
Filters and maps the values in a filterable context with index.
filter_with_index
Filters the values in a filterable context using a predicate with index.
fold_left
Folds a structure from the left.
fold_left_with_index
Folds a structure from the left with index.
fold_map
Maps values to a monoid and combines them.
fold_map_with_index
Maps values with their index to a monoid and combines them.
fold_right
Folds a structure from the right.
fold_right_with_index
Folds a structure from the right with index.
join
Removes one layer of monadic nesting.
lift2
Lifts a binary function into a functor context.
lift3
Lifts a ternary function into a functor context.
lift4
Lifts a quaternary function into a functor context.
lift5
Lifts a quinary function into a functor context.
map
Maps a function over the values in a functor context.
map_first
Maps a function over the first type argument of a bifunctor.
map_second
Maps a function over the second type argument of a bifunctor.
map_with_index
Maps a function with index over the values in a functor context.
partition
Partitions the values in a filterable context using a predicate.
partition_map
Partitions the values in a filterable context using a function that returns Result.
partition_map_with_index
Partitions the values in a filterable context using a function with index that returns Result.
partition_with_index
Partitions the values in a filterable context using a predicate with index.
separate
Separates a container of Result values into two containers.
traverse
Traverses a structure, mapping each element to a computation and combining the results.
traverse_with_index
Traverses a structure with an indexed effectful function, combining the results.
wilt
Partitions a structure based on a function returning a Result in an applicative context.
wither
Maps a function over a data structure and filters out None results in an applicative context.