pub struct Dual<A>(pub A);Expand description
A newtype wrapper that reverses the order of a Semigroup’s operation.
append(Dual(a), Dual(b)) is equivalent to Dual(append(b, a)).
If the inner type is a Monoid, Dual is also a Monoid with the
same identity element.
Examples
use fp_library::{
functions::*,
types::Dual,
};
let x = Dual("ab".to_string());
let y = Dual("cd".to_string());
assert_eq!(append(x, y), Dual("cdab".to_string()));Tuple Fields
0: AThe wrapped value.