pub struct Last<A>(pub A);Expand description
A newtype wrapper whose Semigroup instance always keeps the last value.
append(Last(a), Last(b)) returns Last(b), discarding a.
There is no Monoid instance because there is no identity element.
Examples
use fp_library::{
functions::*,
types::Last,
};
assert_eq!(append(Last("hello"), Last("world")), Last("world"));Tuple Fields
0: AThe wrapped value.