Composed

Struct Composed 

pub struct Composed<'a, S, T, M, N, A, B, O1, O2> {
    pub first: O1,
    pub second: O2,
    /* private fields */
}
This struct represents the composition of two optics, allowing them to be combined into a single optic that applies both transformations.

Type Parameters

  • 'a: The lifetime of the values.
  • S: The source type of the outer structure.
  • T: The target type of the outer structure.
  • M: The source type of the intermediate structure.
  • N: The target type of the intermediate structure.
  • A: The source type of the focus.
  • B: The target type of the focus.
  • O1: The first optic.
  • O2: The second optic.

Fields

first: O1

The outer optic (applied second).

second: O2

The inner optic (applied first).

Implementations

Source

impl<'a, S, T, M, N, A, B, O1, O2> Composed<'a, S, T, M, N, A, B, O1, O2>

Type Parameters
  • 'a: The lifetime of the values.
  • S: The source type of the outer structure.
  • T: The target type of the outer structure.
  • M: The source type of the intermediate structure.
  • N: The target type of the intermediate structure.
  • A: The source type of the focus.
  • B: The target type of the focus.
  • O1: The first optic.
  • O2: The second optic.
pub fn new(first: O1, second: O2) -> Self

Create a new composed optic.

Type Signature

forall S T M N A B O1 O2. (O1, O2) -> Composed S T M N A B O1 O2

Parameters
  • first: The outer optic (applied second).
  • second: The inner optic (applied first).
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::{
		optics::*,
		*,
	},
	classes::optics::*,
	functions::*,
	types::optics::*,
};

let l1: LensPrime<RcBrand, (i32, String), i32> =
	LensPrime::from_view_set(|(x, _): (i32, String)| x, |((_, s), x)| (x, s));
let l2: LensPrime<RcBrand, i32, i32> = LensPrime::from_view_set(|x: i32| x, |(_, x)| x);
let composed = Composed::new(l1, l2);

let f = cloneable_fn_new::<RcFnBrand, _, _>(|x: i32| x * 2);
let modifier = <Composed<
	'_,
	(i32, String),
	(i32, String),
	i32,
	i32,
	i32,
	i32,
	LensPrime<RcBrand, (i32, String), i32>,
	LensPrime<RcBrand, i32, i32>,
> as Optic<RcFnBrand, _, _, _, _>>::evaluate(&composed, f);
assert_eq!(modifier((21, "hi".to_string())), (42, "hi".to_string()));