Forget

Struct Forget 

pub struct Forget<'a, PointerBrand, R, A, B>(pub <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, A, R>, _)
where
    PointerBrand: UnsizedCoercible,
    R: 'a,
    A: 'a;
The Forget profunctor.

Forget<P, R, A, B> is a profunctor that ignores its second type argument B and instead stores a function from A to R.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The pointer brand.
  • R: The return type of the function.
  • A: The input type of the function.
  • B: The ignored type.

Tuple Fields

0: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, A, R>

Implementations

Source

impl<'a, PointerBrand, R, A, B> Forget<'a, PointerBrand, R, A, B>
where PointerBrand: UnsizedCoercible, R: 'a, A: 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The pointer brand.
  • R: The return type of the function.
  • A: The input type of the function.
  • B: The ignored type.
pub fn new(f: impl Fn(A) -> R + 'a) -> Self

Creates a new Forget instance.

Type Signature

forall PointerBrand R A B. UnsizedCoercible PointerBrand => (A -> R) -> Forget PointerBrand R A B

Parameters
  • f: The function to wrap.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Forget,
};

let forget = Forget::<RcBrand, i32, String, i32>::new(|s: String| s.len() as i32);
// Access via the underlying function wrapper, which implements Deref
assert_eq!((forget.0)("hello".to_string()), 5);
pub fn run(&self, a: A) -> R

Runs the Forget profunctor on an input.

Type Signature

forall PointerBrand R A B. UnsizedCoercible PointerBrand => (&Forget PointerBrand R A B, A) -> R

Parameters
  • &self: The forget instance.
  • a: The input value.
Returns

The result of applying the underlying function to the input.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Forget,
};

let forget = Forget::<RcBrand, i32, String, i32>::new(|s: String| s.len() as i32);
assert_eq!(forget.run("hello".to_string()), 5);