Getter

Struct Getter 

pub struct Getter<'a, PointerBrand, S, T, A, B>
where PointerBrand: UnsizedCoercible, S: 'a, T: 'a, A: 'a, B: 'a,
{ pub view_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, S, A>, /* private fields */ }
A polymorphic getter.

Matches PureScript’s Getter s t a b.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure.
  • A: The source type of the focus.
  • B: The target type of the focus.

Fields

view_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, S, A>

Function to view the focus of the getter in a structure.

Implementations

Source

impl<'a, PointerBrand, S, T, A, B> Getter<'a, PointerBrand, S, T, A, B>
where PointerBrand: UnsizedCoercible, S: 'a, T: 'a, A: 'a, B: 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure.
  • A: The source type of the focus.
  • B: The target type of the focus.
pub fn new(view: impl 'a + Fn(S) -> A) -> Self

Create a new getter from a view function.

Type Signature

forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (S -> A) -> Getter PointerBrand S T A B

Parameters
  • view: The view function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Getter,
};

let g: Getter<RcBrand, (i32, String), (i32, String), i32, i32> = Getter::new(|(x, _)| x);
assert_eq!(g.view((42, "hi".to_string())), 42);
pub fn view(&self, s: S) -> A

View the focus of the getter in a structure.

Type Signature

forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (&Getter PointerBrand S T A B, S) -> A

Parameters
  • &self: The getter instance.
  • s: The structure to view.
Returns

The focus value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Getter,
};

let g: Getter<RcBrand, (i32, String), (i32, String), i32, i32> = Getter::new(|(x, _)| x);
assert_eq!(g.view((42, "hi".to_string())), 42);