GetterPrime

Struct GetterPrime 

pub struct GetterPrime<'a, PointerBrand, S, A>
where PointerBrand: UnsizedCoercible, S: 'a, A: 'a,
{ pub view_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, S, A>, /* private fields */ }
A concrete getter type where types do not change.

Matches PureScript’s Getter' s a.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.

Fields

view_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, S, A>

Function to view the focus of the getter in a structure.

Implementations

Source

impl<'a, PointerBrand, S, A> GetterPrime<'a, PointerBrand, S, A>
where PointerBrand: UnsizedCoercible, S: 'a, A: 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.
pub fn new(view: impl 'a + Fn(S) -> A) -> Self

Create a new monomorphic getter from a view function.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (S -> A) -> GetterPrime PointerBrand S A

Parameters
  • view: The view function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::GetterPrime,
};

let g: GetterPrime<RcBrand, (i32, String), i32> = GetterPrime::new(|(x, _)| x);
assert_eq!(g.view((42, "hi".to_string())), 42);
pub fn view(&self, s: S) -> A

View the focus of the getter in a structure.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (&GetterPrime PointerBrand S A, S) -> A

Parameters
  • &self: The getter instance.
  • s: The structure to view.
Returns

The focus value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::GetterPrime,
};

let g: GetterPrime<RcBrand, (i32, String), i32> = GetterPrime::new(|(x, _)| x);
assert_eq!(g.view((42, "hi".to_string())), 42);