Grate

Struct Grate 

pub struct Grate<'a, PointerBrand, S, T, A, B>
where PointerBrand: ToDynCloneFn, S: 'a, T: 'a, A: 'a, B: 'a,
{ pub grate: <FnBrand<PointerBrand> as CloneFn>::Of<'a, <FnBrand<PointerBrand> as CloneFn>::Of<'a, <FnBrand<PointerBrand> as CloneFn>::Of<'a, <PointerBrand as RefCountedPointer>::Of<'a, S>, A>, B>, T>, }
A polymorphic grate.

Matches PureScript’s Grate s t a b.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.

Fields

grate: <FnBrand<PointerBrand> as CloneFn>::Of<'a, <FnBrand<PointerBrand> as CloneFn>::Of<'a, <FnBrand<PointerBrand> as CloneFn>::Of<'a, <PointerBrand as RefCountedPointer>::Of<'a, S>, A>, B>, T>

Grating function.

Implementations

Source

impl<'a, PointerBrand, S, T, A, B> Grate<'a, PointerBrand, S, T, A, B>
where PointerBrand: ToDynCloneFn, S: 'a, T: 'a, A: 'a, B: 'a + Clone,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.
pub fn new( grate: impl Fn(<FnBrand<PointerBrand> as CloneFn>::Of<'a, <FnBrand<PointerBrand> as CloneFn>::Of<'a, <PointerBrand as RefCountedPointer>::Of<'a, S>, A>, B>) -> T + 'a, ) -> Self
where <PointerBrand as RefCountedPointer>::Of<'a, S>: Sized,

Creates a new Grate instance.

Type Signature

forall PointerBrand S T A B. ToDynCloneFn PointerBrand => (Fn PointerBrand (Fn PointerBrand (PointerBrand S) A) B -> T) -> Grate PointerBrand S T A B

Parameters
  • grate: The grating function.
Returns

A new instance of the type.

Examples
use {
	fp_library::{
		brands::*,
		classes::*,
		types::optics::Grate,
	},
	std::rc::Rc,
};

let grate = Grate::<'_, RcBrand, (i32, i32), (i32, i32), i32, i32>::new(|f| {
	let get_x = <RcFnBrand as LiftFn>::new(|s: Rc<(i32, i32)>| s.0);
	let get_y = <RcFnBrand as LiftFn>::new(|s: Rc<(i32, i32)>| s.1);
	(f(get_x), f(get_y))
});
assert_eq!(grate.zip_with(|(a, b)| a + b, (1, 2), (3, 4)), (4, 6));
pub fn zip_with(&self, f: impl Fn((A, A)) -> B + 'a, s1: S, s2: S) -> T
where <PointerBrand as RefCountedPointer>::Of<'a, S>: Sized,

Zip two structures together using this grate and a combining function.

Convenience method wrapping zip_with_of.

Type Signature

forall PointerBrand S T A B. ToDynCloneFn PointerBrand => (&Grate PointerBrand S T A B, (A, A) -> B, S, S) -> T

Parameters
  • &self: The grate instance.
  • f: The combining function, taking a pair (A, A) and returning B.
  • s1: The first structure.
  • s2: The second structure.
Returns

The combined structure.

Examples
use {
	fp_library::{
		brands::*,
		classes::*,
		types::optics::Grate,
	},
	std::rc::Rc,
};

let grate = Grate::<'_, RcBrand, (i32, i32), (i32, i32), i32, i32>::new(|f| {
	let get_x = <RcFnBrand as LiftFn>::new(|s: Rc<(i32, i32)>| s.0);
	let get_y = <RcFnBrand as LiftFn>::new(|s: Rc<(i32, i32)>| s.1);
	(f(get_x), f(get_y))
});
let result = grate.zip_with(|(a, b)| a + b, (1, 2), (10, 20));
assert_eq!(result, (11, 22));