IndexedTraversal

Struct IndexedTraversal 

pub struct IndexedTraversal<'a, PointerBrand, I, S, T, A, B, F>
where F: IndexedTraversalFunc<'a, I, S, T, A, B> + 'a,
{ pub traversal: F, /* private fields */ }
A polymorphic indexed traversal.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • I: The index type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.
  • F: The traversal function type.

Fields

traversal: F

The underlying indexed traversal function.

Implementations

Source

impl<'a, PointerBrand, I, Brand, A, B> IndexedTraversal<'a, PointerBrand, I, <Brand as Kind_cdc7cd43dac7585f>::Of<'a, A>, <Brand as Kind_cdc7cd43dac7585f>::Of<'a, B>, A, B, Traversed<Brand>>
where Brand: TraversableWithIndex<Index = I>, A: 'a + Clone, B: 'a + Clone, I: 'a, <Brand as Kind_cdc7cd43dac7585f>::Of<'a, B>: Clone,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The profunctor type.
  • I: The index type.
  • Brand: The brand of the traversable structure.
  • A: The type of the elements in the structure.
  • B: The type of the elements in the result.
pub fn traversed() -> Self

Create an indexed traversal from a TraversableWithIndex.

Type Signature

forall PointerBrand I Brand A B. TraversableWithIndex Brand => () -> IndexedTraversal PointerBrand I Brand A B

Returns

A new IndexedTraversal instance.

Examples
use fp_library::{
	brands::{
		RcBrand,
		VecBrand,
		optics::*,
	},
	functions::optics_indexed_fold_map,
	types::optics::*,
};
let t: IndexedTraversal<RcBrand, usize, Vec<i32>, Vec<i32>, i32, i32, Traversed<VecBrand>> =
	IndexedTraversal::traversed();
let v = vec![10, 20, 30];
let s =
	optics_indexed_fold_map::<RcBrand, _, _, _, String>(&t, |i, x| format!("{}:{}", i, x), v);
assert_eq!(s, "0:101:202:30".to_string());
Source

impl<'a, PointerBrand, I, S, T, A, B, F> IndexedTraversal<'a, PointerBrand, I, S, T, A, B, F>
where F: IndexedTraversalFunc<'a, I, S, T, A, B> + 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • I: The index type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.
  • F: The traversal function type.
pub fn new(traversal: F) -> Self

Create a new indexed traversal.

Type Signature

forall PointerBrand I S T A B F. IndexedTraversalFunc F => F -> IndexedTraversal PointerBrand I S T A B F

Parameters
  • traversal: The traversal function.
Returns

A new IndexedTraversal instance.

Examples
use fp_library::{
	brands::{
		RcBrand,
		RcFnBrand,
		VecBrand,
		optics::*,
	},
	classes::optics::IndexedTraversalOptic,
	types::optics::*,
};
let l: IndexedTraversal<RcBrand, usize, Vec<i32>, Vec<i32>, i32, i32, Traversed<VecBrand>> =
	IndexedTraversal::new(Traversed(std::marker::PhantomData));
let f = std::rc::Rc::new(|(i, x): (usize, i32)| x + (i as i32))
	as std::rc::Rc<dyn Fn((usize, i32)) -> i32>;
let pab = Indexed::new(f);
let result: std::rc::Rc<dyn Fn(Vec<i32>) -> Vec<i32>> =
	IndexedTraversalOptic::evaluate::<RcFnBrand>(&l, pab);
assert_eq!(result(vec![10, 20]), vec![10, 21]);