Iso

Struct Iso 

pub struct Iso<'a, PointerBrand, S, T, A, B>
where PointerBrand: UnsizedCoercible, S: 'a, T: 'a, A: 'a, B: 'a,
{ pub from: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, S, A>, pub to: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, B, T>, }
A polymorphic isomorphism where types can change. This matches PureScript’s Iso s t a b.

An isomorphism represents a lossless bidirectional conversion between types. Uses FnBrand to support capturing closures.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.

Fields

from: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, S, A>

Forward conversion: from structure to focus.

to: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, B, T>

Backward conversion: from focus to structure.

Implementations

Source

impl<'a, PointerBrand, S: 'a, T: 'a, A: 'a, B: 'a> Iso<'a, PointerBrand, S, T, A, B>
where PointerBrand: UnsizedCoercible,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.
pub fn new(from: impl 'a + Fn(S) -> A, to: impl 'a + Fn(B) -> T) -> Self

Create a new polymorphic isomorphism.

Type Signature

forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (S -> A, B -> T) -> Iso PointerBrand S T A B

Parameters
  • from: The forward conversion function.
  • to: The backward conversion function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Iso,
};

// Iso between String and Vec<char>
let string_chars: Iso<RcBrand, String, String, Vec<char>, Vec<char>> =
	Iso::new(|s: String| s.chars().collect(), |v: Vec<char>| v.into_iter().collect());
assert_eq!(string_chars.from("hi".to_string()), vec!['h', 'i']);
pub fn from(&self, s: S) -> A

Apply the forward conversion.

Type Signature

forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (&Iso PointerBrand S T A B, S) -> A

Parameters
  • &self: The iso instance.
  • s: The structure to convert.
Returns

The focus value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Iso,
};

let string_chars: Iso<RcBrand, String, String, Vec<char>, Vec<char>> =
	Iso::new(|s: String| s.chars().collect(), |v: Vec<char>| v.into_iter().collect());
let chars = string_chars.from("hello".to_string());
assert_eq!(chars, vec!['h', 'e', 'l', 'l', 'o']);
pub fn to(&self, b: B) -> T

Apply the backward conversion.

Type Signature

forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (&Iso PointerBrand S T A B, B) -> T

Parameters
  • &self: The iso instance.
  • b: The focus value to convert.
Returns

The structure value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Iso,
};

let string_chars: Iso<RcBrand, String, String, Vec<char>, Vec<char>> =
	Iso::new(|s: String| s.chars().collect(), |v: Vec<char>| v.into_iter().collect());
let s = string_chars.to(vec!['h', 'i']);
assert_eq!(s, "hi");