IsoPrime

Struct IsoPrime 

pub struct IsoPrime<'a, PointerBrand, S, A>
where PointerBrand: ToDynCloneFn, S: 'a, A: 'a,
{ /* private fields */ }
A concrete isomorphism type where types do not change. This matches PureScript’s Iso' s a.

Uses FnBrand to support capturing closures.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.

Implementations

Source

impl<'a, PointerBrand, S: 'a, A: 'a> IsoPrime<'a, PointerBrand, S, A>
where PointerBrand: ToDynCloneFn,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.
pub fn new(from: impl 'a + Fn(S) -> A, to: impl 'a + Fn(A) -> S) -> Self

Create a new monomorphic isomorphism from bidirectional conversion functions.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => (S -> A, A -> S) -> IsoPrime PointerBrand S A

Parameters
  • from: The forward conversion function.
  • to: The backward conversion function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::IsoPrime,
};

// Iso between a newtype and its inner value
let iso: IsoPrime<RcBrand, (i32,), i32> = IsoPrime::new(|(x,)| x, |x| (x,));
assert_eq!(iso.from((10,)), 10);
pub fn from(&self, s: S) -> A

Apply the forward conversion.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => (&IsoPrime PointerBrand S A, S) -> A

Parameters
  • &self: The monomorphic iso instance.
  • s: The structure to convert.
Returns

The focus value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::IsoPrime,
};

let iso: IsoPrime<RcBrand, (i32,), i32> = IsoPrime::new(|(x,)| x, |x| (x,));
assert_eq!(iso.from((42,)), 42);
pub fn to(&self, a: A) -> S

Apply the backward conversion.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => (&IsoPrime PointerBrand S A, A) -> S

Parameters
  • &self: The monomorphic iso instance.
  • a: The focus value to convert.
Returns

The structure value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::IsoPrime,
};

let iso: IsoPrime<RcBrand, (i32,), i32> = IsoPrime::new(|(x,)| x, |x| (x,));
assert_eq!(iso.to(42), (42,));
pub fn reversed(&self) -> IsoPrime<'a, PointerBrand, A, S>

Reverse the isomorphism.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => &IsoPrime PointerBrand S A -> IsoPrime PointerBrand A S

Returns

A new IsoPrime instance with the forward and backward conversions swapped.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::IsoPrime,
};

let iso: IsoPrime<RcBrand, (i32,), i32> = IsoPrime::new(|(x,)| x, |x| (x,));
let reversed = iso.reversed();
assert_eq!(reversed.from(42), (42,));
assert_eq!(reversed.to((42,)), 42);