LensPrime

Struct LensPrime 

pub struct LensPrime<'a, PointerBrand, S, A>
where PointerBrand: UnsizedCoercible, S: 'a, A: 'a,
{ /* private fields */ }
A concrete lens type for accessing and updating a field in a structure where types do not change. This matches PureScript’s Lens' s a.

Uses FnBrand to support capturing closures.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.

Implementations

Source

impl<'a, PointerBrand, S: 'a, A: 'a> LensPrime<'a, PointerBrand, S, A>
where PointerBrand: UnsizedCoercible,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.
pub fn new( to: impl 'a + Fn(S) -> (A, <FnBrand<PointerBrand> as CloneableFn>::Of<'a, A, S>), ) -> Self

Create a new monomorphic lens. This matches PureScript’s lens' constructor.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (S -> (A, A -> S)) -> LensPrime PointerBrand S A

Parameters
  • to: The getter/setter pair function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::{
		RcBrand,
		RcFnBrand,
	},
	classes::CloneableFn,
	types::optics::LensPrime,
};

let l: LensPrime<RcBrand, i32, i32> =
	LensPrime::new(|x| (x, <RcFnBrand as CloneableFn>::new(|s| s)));
assert_eq!(l.view(42), 42);
pub fn from_view_set( view: impl 'a + Fn(S) -> A, set: impl 'a + Fn((S, A)) -> S, ) -> Self
where S: Clone,

Create a new monomorphic lens from a getter and setter.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (S -> A, (S, A) -> S) -> LensPrime PointerBrand S A

Parameters
  • view: The getter function.
  • set: The setter function.
Returns

A new LensPrime instance.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::LensPrime,
};

let l: LensPrime<RcBrand, i32, i32> = LensPrime::from_view_set(|x: i32| x, |(_, y)| y);
assert_eq!(l.view(10), 10);
pub fn view(&self, s: S) -> A

View the focus of the lens in a structure.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (&LensPrime PointerBrand S A, S) -> A

Parameters
  • &self: The monomorphic lens instance.
  • s: The structure to view.
Returns

The focus value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::LensPrime,
};

let l: LensPrime<RcBrand, i32, i32> = LensPrime::from_view_set(|x: i32| x, |(_, y)| y);
assert_eq!(l.view(42), 42);
pub fn set(&self, s: S, a: A) -> S

Set the focus of the lens in a structure.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (&LensPrime PointerBrand S A, S, A) -> S

Parameters
  • &self: The monomorphic lens instance.
  • s: The structure to update.
  • a: The new value for the focus.
Returns

The updated structure.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::LensPrime,
};

let l: LensPrime<RcBrand, i32, i32> = LensPrime::from_view_set(|x: i32| x, |(_, y)| y);
assert_eq!(l.set(10, 20), 20);
pub fn over(&self, s: S, f: impl Fn(A) -> A) -> S

Update the focus of the lens in a structure using a function.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (&LensPrime PointerBrand S A, S, A -> A) -> S

Parameters
  • &self: The monomorphic lens instance.
  • s: The structure to update.
  • f: The function to apply to the focus.
Returns

The updated structure.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::LensPrime,
};

let l: LensPrime<RcBrand, i32, i32> = LensPrime::from_view_set(|x: i32| x, |(_, y)| y);
assert_eq!(l.over(10, |x| x + 1), 11);