PrismPrime

Struct PrismPrime 

pub struct PrismPrime<'a, PointerBrand, S, A>
where PointerBrand: ToDynCloneFn, S: 'a, A: 'a,
{ /* private fields */ }
A concrete prism type for sum types where types do not change. This matches PureScript’s Prism' s a.

Uses FnBrand to support capturing closures.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.

Implementations

Source

impl<'a, PointerBrand, S: 'a, A: 'a> PrismPrime<'a, PointerBrand, S, A>
where PointerBrand: ToDynCloneFn,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.
pub fn new( preview: impl 'a + Fn(S) -> Result<A, S>, review: impl 'a + Fn(A) -> S, ) -> Self

Create a new monomorphic prism from preview and review functions without requiring S: Clone. This uses a Result<A, S> which is closer to the true Either encoding.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => (S -> Result A S, A -> S) -> PrismPrime PointerBrand S A

Parameters
  • preview: The preview function.
  • review: The review function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::PrismPrime,
};

let ok_prism: PrismPrime<RcBrand, Result<i32, String>, i32> =
	PrismPrime::new(|r: Result<i32, String>| r.map_err(|e| Err(e)), |x| Ok(x));
assert_eq!(ok_prism.preview(Ok(42)), Some(42));
pub fn from_option( preview: impl 'a + Fn(S) -> Option<A>, review: impl 'a + Fn(A) -> S, ) -> Self
where S: Clone,

Create a new monomorphic prism from preview and review functions.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => (S -> Option A, A -> S) -> PrismPrime PointerBrand S A

Parameters
  • preview: The preview function.
  • review: The review function.
Returns

A new PrismPrime instance.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::PrismPrime,
};

let ok_prism: PrismPrime<RcBrand, Result<i32, String>, i32> =
	PrismPrime::from_option(|r: Result<i32, String>| r.ok(), |x| Ok(x));
assert_eq!(ok_prism.preview(Ok(42)), Some(42));
pub fn preview(&self, s: S) -> Option<A>

Preview the focus of the prism in a structure.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => (&PrismPrime PointerBrand S A, S) -> Option A

Parameters
  • &self: The monomorphic prism instance.
  • s: The structure to preview.
Returns

The focus value if it exists, or None if not.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::PrismPrime,
};

let ok_prism: PrismPrime<RcBrand, Result<i32, String>, i32> =
	PrismPrime::from_option(|r: Result<i32, String>| r.ok(), |x| Ok(x));
assert_eq!(ok_prism.preview(Ok(42)), Some(42));
assert_eq!(ok_prism.preview(Err("error".to_string())), None);
pub fn review(&self, a: A) -> S

Review the focus into the structure.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => (&PrismPrime PointerBrand S A, A) -> S

Parameters
  • &self: The monomorphic prism instance.
  • a: The focus value.
Returns

The structure containing the focus value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::PrismPrime,
};

let ok_prism: PrismPrime<RcBrand, Result<i32, String>, i32> =
	PrismPrime::from_option(|r: Result<i32, String>| r.ok(), |x| Ok(x));
assert_eq!(ok_prism.review(42), Ok(42));
pub fn modify(&self, s: S, f: impl Fn(A) -> A) -> S

Modify the focus if it exists.

Type Signature

forall PointerBrand S A. ToDynCloneFn PointerBrand => (&PrismPrime PointerBrand S A, S, A -> A) -> S

Parameters
  • &self: The monomorphic prism instance.
  • s: The structure to update.
  • f: The function to apply to the focus.
Returns

The updated structure.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::PrismPrime,
};

let ok_prism: PrismPrime<RcBrand, Result<i32, String>, i32> =
	PrismPrime::from_option(|r: Result<i32, String>| r.ok(), |x| Ok(x));
assert_eq!(ok_prism.modify(Ok(21), |x| x * 2), Ok(42));
assert_eq!(ok_prism.modify(Err("error".to_string()), |x| x * 2), Err("error".to_string()));