ReversedOptic

Struct ReversedOptic 

pub struct ReversedOptic<'a, PointerBrand, S, T, A, B, O>
where PointerBrand: ToDynCloneFn, S: 'a, T: 'a, A: 'a, B: 'a,
{ /* private fields */ }
A reversed optic, produced by the reverse combinator.

ReversedOptic wraps an inner optic and implements reversed optic traits by evaluating the inner optic with ReverseBrand<ConcreteP> as the profunctor.

Corresponds to PureScript’s re :: Optic (Re p a b) s t a b -> Optic p b a t s.

The reversed optic swaps the roles of source/target and focus types:

  • An optic S -> T, A -> B becomes B -> A, T -> S (for review)
  • A simple optic S <-> A becomes A <-> S (for getter/fold)

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The cloneable function pointer brand used by the Reverse profunctor.
  • S: The source type of the original optic.
  • T: The target type of the original optic.
  • A: The focus source type of the original optic.
  • B: The focus target type of the original optic.
  • O: The inner optic type.