Review

Struct Review 

pub struct Review<'a, PointerBrand, S, T, A, B>
where PointerBrand: UnsizedCoercible, S: 'a, T: 'a, A: 'a, B: 'a,
{ pub review_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, B, T>, /* private fields */ }
A polymorphic review.

Matches PureScript’s Review s t a b.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure.
  • A: The source type of the focus.
  • B: The target type of the focus.

Fields

review_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, B, T>

Function to construct a structure from a focus value.

Implementations

Source

impl<'a, PointerBrand, S, T, A, B> Review<'a, PointerBrand, S, T, A, B>
where PointerBrand: UnsizedCoercible, S: 'a, T: 'a, A: 'a, B: 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure.
  • A: The source type of the focus.
  • B: The target type of the focus.
pub fn new(review: impl 'a + Fn(B) -> T) -> Self

Create a new polymorphic review from a review function.

Type Signature

forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (B -> T) -> Review PointerBrand S T A B

Parameters
  • review: The review function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Review,
};

let r: Review<RcBrand, Option<i32>, Option<i32>, i32, i32> = Review::new(Some);
assert_eq!(r.review(42), Some(42));
pub fn review(&self, b: B) -> T

Review a focus value into a structure.

Type Signature

forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (&Review PointerBrand S T A B, B) -> T

Parameters
  • &self: The review instance.
  • b: The focus value to review.
Returns

The structure containing the focus value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::Review,
};

let r: Review<RcBrand, Option<i32>, Option<i32>, i32, i32> = Review::new(Some);
assert_eq!(r.review(42), Some(42));