ReviewPrime

Struct ReviewPrime 

pub struct ReviewPrime<'a, PointerBrand, S, A>
where PointerBrand: UnsizedCoercible, S: 'a, A: 'a,
{ pub review_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, A, S>, /* private fields */ }
A concrete review type where types do not change.

Matches PureScript’s Review' s a.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.

Fields

review_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, A, S>

Function to construct a structure from a focus value.

Implementations

Source

impl<'a, PointerBrand, S, A> ReviewPrime<'a, PointerBrand, S, A>
where PointerBrand: UnsizedCoercible, S: 'a, A: 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The type of the structure.
  • A: The type of the focus.
pub fn new(review: impl 'a + Fn(A) -> S) -> Self

Create a new monomorphic review.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (A -> S) -> ReviewPrime PointerBrand S A

Parameters
  • review: The review function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::ReviewPrime,
};

let r: ReviewPrime<RcBrand, Option<i32>, i32> = ReviewPrime::new(Some);
assert_eq!(r.review(42), Some(42));
pub fn review(&self, a: A) -> S

Review a focus value into a structure.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (&ReviewPrime PointerBrand S A, A) -> S

Parameters
  • &self: The review instance.
  • a: The focus value to review.
Returns

The structure containing the focus value.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::ReviewPrime,
};

let r: ReviewPrime<RcBrand, Option<i32>, i32> = ReviewPrime::new(Some);
assert_eq!(r.review(42), Some(42));