Traversal

Struct Traversal 

pub struct Traversal<'a, PointerBrand, S, T, A, B, F>
where PointerBrand: ToDynCloneFn, F: TraversalFunc<'a, S, T, A, B> + 'a, S: 'a, T: 'a, A: 'a, B: 'a,
{ pub traversal: F, /* private fields */ }
A polymorphic traversal.

Matches PureScript’s Traversal s t a b.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.
  • F: The type of the traversal function.

Fields

traversal: F

The traversal function.

In PureScript this is (forall f. Applicative f => (a -> f b) -> s -> f t).

Implementations

Source

impl<'a, PointerBrand, S, T, A, B, F> Traversal<'a, PointerBrand, S, T, A, B, F>
where PointerBrand: ToDynCloneFn, F: TraversalFunc<'a, S, T, A, B> + 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.
  • F: The type of the traversal function.
pub fn new(traversal: F) -> Self

Creates a new Traversal instance.

Type Signature

forall PointerBrand S T A B F. (ToDynCloneFn PointerBrand, TraversalFunc F) => F -> Traversal PointerBrand S T A B F

Parameters
  • traversal: The traversal function.
Returns

A new instance of the type.

Examples
use fp_library::{
	Apply,
	brands::*,
	classes::{
		Applicative,
		lift::Lift,
		optics::traversal::TraversalFunc,
		pointed::Pointed,
	},
	kinds::*,
	types::optics::Traversal,
};

#[derive(Clone)]
struct ListTraversal;
impl<'a, A: 'a + Clone> TraversalFunc<'a, Vec<A>, Vec<A>, A, A> for ListTraversal {
	fn apply<M: Applicative>(
		&self,
		f: Box<dyn Fn(A) -> Apply!(<M as Kind!( type Of<'b, U: 'b>: 'b; )>::Of<'a, A>) + 'a>,
		s: Vec<A>,
	) -> Apply!(<M as Kind!( type Of<'b, U: 'b>: 'b; )>::Of<'a, Vec<A>>) {
		s.into_iter().fold(M::pure(vec![]), |acc, a| {
			M::lift2(
				|mut v: Vec<A>, x: A| {
					v.push(x);
					v
				},
				acc,
				f(a),
			)
		})
	}
}

let traversal = Traversal::<'_, RcBrand, Vec<i32>, Vec<i32>, i32, i32, _>::new(ListTraversal);
assert_eq!(
	traversal.traversal.apply::<OptionBrand>(Box::new(|x| Some(x + 1)), vec![1, 2]),
	Some(vec![2, 3])
);