Module explicit

Module explicit 

Explicit dispatch functions requiring a Brand turbofish.

For most use cases, prefer the inference-enabled wrappers from functions.

Functions

filter
Filters the values in a filterable context using a predicate.
filter_map
Filters and maps the values in a filterable context.
partition
Partitions the values in a filterable context using a predicate.
partition_map
Partitions the values in a filterable context using a function that returns Result.