pub trait ClosureMode {
type Target<'a, A: 'a, B: 'a>: ?Sized + 'a;
type SendTarget<'a, A: 'a, B: 'a>: ?Sized + 'a;
}Expand description
Required Associated Types
Sourcetype Target<'a, A: 'a, B: 'a>: ?Sized + 'a
type Target<'a, A: 'a, B: 'a>: ?Sized + 'a
The unsized closure trait object type for this mode.
Sourcetype SendTarget<'a, A: 'a, B: 'a>: ?Sized + 'a
type SendTarget<'a, A: 'a, B: 'a>: ?Sized + 'a
The unsized closure trait object type for this mode with Send + Sync bounds.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.