Explicit dispatch functions requiring a Brand turbofish.
For most use cases, prefer the inference-enabled wrappers from the parent
functions module.
Functions
- alt
- Combines two values in a context, choosing associatively.
- apply_
first - Sequences two applicative actions, keeping the result of the first.
- apply_
second - Sequences two applicative actions, keeping the result of the second.
- bi_
fold_ left - Folds a bifoldable structure from the left.
- bi_
fold_ map - Maps elements of both types to a monoid and combines them.
- bi_
fold_ right - Folds a bifoldable structure from the right.
- bi_
traverse - Traverses a bitraversable structure, mapping each element to a computation and combining the results.
- bimap
- Maps two functions over the values in a bifunctor context.
- bind
- Sequences a monadic computation with a function that produces the next computation.
- bind_
flipped - Binds a monadic value to a function (flipped argument order).
- compact
- Removes
Nonevalues from a container ofOptions, unwrapping theSomevalues. - contramap
- Contravariantly maps a function over a contravariant functor.
- filter
- Filters the values in a filterable context using a predicate.
- filter_
map - Filters and maps the values in a filterable context.
- filter_
map_ with_ index - Filters and maps the values in a filterable context with index.
- filter_
with_ index - Filters the values in a filterable context using a predicate with index.
- fold_
left - Folds a structure from the left.
- fold_
left_ with_ index - Folds a structure from the left with index.
- fold_
map - Maps values to a monoid and combines them.
- fold_
map_ with_ index - Maps values with their index to a monoid and combines them.
- fold_
right - Folds a structure from the right.
- fold_
right_ with_ index - Folds a structure from the right with index.
- join
- Removes one layer of monadic nesting.
- lift2
- Lifts a binary function into a functor context.
- lift3
- Lifts a ternary function into a functor context.
- lift4
- Lifts a quaternary function into a functor context.
- lift5
- Lifts a quinary function into a functor context.
- map
- Maps a function over the values in a functor context.
- map_
with_ index - Maps a function with index over the values in a functor context.
- partition
- Partitions the values in a filterable context using a predicate.
- partition_
map - Partitions the values in a filterable context using a function that returns
Result. - partition_
map_ with_ index - Partitions the values in a filterable context using a function with index that returns
Result. - partition_
with_ index - Partitions the values in a filterable context using a predicate with index.
- separate
- Separates a container of
Resultvalues into two containers. - traverse
- Traverses a structure, mapping each element to a computation and combining the results.
- traverse_
with_ index - Traverses a structure with an indexed effectful function, combining the results.
- wilt
- Partitions a structure based on a function returning a Result in an applicative context.
- wither
- Maps a function over a data structure and filters out None results in an applicative context.