pub enum FreeStep<F, A>{
Done(A),
Suspended(<F as Kind_cdc7cd43dac7585f>::Of<'static, Free<F, A>>),
}Expand description
The result of stepping through a Free computation.
Produced by Free::to_view, this decomposes a Free into either
a final value or a suspended functor layer with all pending
continuations reattached. This provides a unified, single-step
decomposition that both Free::evaluate and Free::resume
delegate to.
Type Parameters
Variants
Done(A)
The computation completed with a final value.
Suspended(<F as Kind_cdc7cd43dac7585f>::Of<'static, Free<F, A>>)
The computation is suspended in the functor F.
The inner Free values have all pending continuations reattached.