pub struct Identity<A>(pub A);Expand description
Wraps a value.
The Identity type represents a trivial wrapper around a value. It is the simplest possible container.
It is often used as a base case for higher-kinded types or when a container is required but no additional effect is needed.
Higher-Kinded Type Representation
The higher-kinded representation of this type constructor is IdentityBrand,
which is fully polymorphic over the wrapped value type.
Serialization
This type supports serialization and deserialization via serde when the serde feature is enabled.
Type Parameters
A: The type of the wrapped value.
Fields
0: The wrapped value.
Examples
use fp_library::types::*;
let x = Identity(5);
assert_eq!(x.0, 5);Tuple Fields
0: ATrait Implementations
Sourceimpl<'de, A> Deserialize<'de> for Identity<A>where
A: Deserialize<'de>,
impl<'de, A> Deserialize<'de> for Identity<A>where
A: Deserialize<'de>,
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl<A: Ord> Ord for Identity<A>
impl<A: Ord> Ord for Identity<A>
1.21.0 · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl<A: PartialOrd> PartialOrd for Identity<A>
impl<A: PartialOrd> PartialOrd for Identity<A>
impl<A: Copy> Copy for Identity<A>
impl<A: Eq> Eq for Identity<A>
impl<A> StructuralPartialEq for Identity<A>