Bazaar

Struct Bazaar 

pub struct Bazaar<'a, FunctionBrand: CloneableFn + 'a, A: 'a, B: 'a, S: 'a, T: 'a> {
    pub run: <FunctionBrand as CloneableFn>::Of<'a, S, BazaarList<'a, FunctionBrand, A, B, T>>,
}
The Bazaar profunctor, used to characterize traversals.

Wraps a decomposition function from S to BazaarList, which extracts all focus values and provides a rebuild function. Use run_bazaar to interpret the Bazaar with any Applicative.

This is a port of PureScript’s Bazaar ((->) :: Type -> Type -> Type) a b s t, specialized to the function profunctor.

Type Parameters

  • 'a: The lifetime of the values.
  • FunctionBrand: The cloneable function brand.
  • A: The type of focus values extracted from the source.
  • B: The type of replacement values used during reconstruction.
  • S: The source type.
  • T: The target type.

Fields

run: <FunctionBrand as CloneableFn>::Of<'a, S, BazaarList<'a, FunctionBrand, A, B, T>>

Decomposes a source into a BazaarList of foci and a rebuild function.

Implementations

Source

impl<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> Bazaar<'a, FunctionBrand, A, B, S, T>

Type Parameters
  • 'a: The lifetime of the values.
  • FunctionBrand: The cloneable function brand.
  • A: The type of focus values extracted from the source.
  • B: The type of replacement values used during reconstruction.
  • S: The source type.
  • T: The target type.
pub fn new( run: <FunctionBrand as CloneableFn>::Of<'a, S, BazaarList<'a, FunctionBrand, A, B, T>>, ) -> Self

Creates a new Bazaar instance.

Type Signature

forall A B S T. (S -> BazaarList FunctionBrand A B T) -> Bazaar FunctionBrand A B S T

Parameters
  • run: The decomposition function from source to BazaarList.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::{
		optics::*,
		*,
	},
	functions::*,
	types::optics::*,
};

let bazaar = Bazaar::<RcFnBrand, i32, i32, i32, i32>::new(cloneable_fn_new::<RcFnBrand, _, _>(
	|s: i32| BazaarList {
		foci: vec![s],
		rebuild: cloneable_fn_new::<RcFnBrand, _, _>(|bs: Vec<i32>| bs[0]),
	},
));
let bl = (bazaar.run)(42);
assert_eq!(bl.foci, vec![42]);
assert_eq!((bl.rebuild)(vec![100]), 100);