pub struct Exchange<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> {
pub get: <FunctionBrand as CloneableFn>::Of<'a, S, A>,
pub set: <FunctionBrand as CloneableFn>::Of<'a, B, T>,
}Expand description
The Exchange profunctor.
Type Parameters
'a: The lifetime of the functions.FunctionBrand: The cloneable function brand.A: The type of the value produced by the forward function.B: The type of the value consumed by the backward function.S: The source type of the structure.T: The target type of the structure.
Fields
get: <FunctionBrand as CloneableFn>::Of<'a, S, A>Forward function.
set: <FunctionBrand as CloneableFn>::Of<'a, B, T>Backward function.
Implementations
Sourceimpl<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> Exchange<'a, FunctionBrand, A, B, S, T>
Type Parameters
'a: The lifetime of the functions.
FunctionBrand: The cloneable function brand.
A: The type of the value produced by the forward function.
B: The type of the value consumed by the backward function.
S: The source type of the structure.
T: The target type of the structure.
impl<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> Exchange<'a, FunctionBrand, A, B, S, T>
Type Parameters
'a: The lifetime of the functions.FunctionBrand: The cloneable function brand.A: The type of the value produced by the forward function.B: The type of the value consumed by the backward function.S: The source type of the structure.T: The target type of the structure.
Sourcepub fn new(
get: <FunctionBrand as CloneableFn>::Of<'a, S, A>,
set: <FunctionBrand as CloneableFn>::Of<'a, B, T>,
) -> Self
pub fn new( get: <FunctionBrand as CloneableFn>::Of<'a, S, A>, set: <FunctionBrand as CloneableFn>::Of<'a, B, T>, ) -> Self
Creates a new Exchange instance.
Type Signature
forall A B S T. (S -> A, B -> T) -> Exchange FunctionBrand A B S T
Parameters
get: The forward function.set: The backward function.
Returns
A new instance of the type.
Examples
use fp_library::{
brands::RcFnBrand,
classes::cloneable_fn::new as cloneable_fn_new,
types::optics::Exchange,
};
let exchange = Exchange::<RcFnBrand, _, _, _, _>::new(
cloneable_fn_new::<RcFnBrand, _, _>(|s: String| s.len()),
cloneable_fn_new::<RcFnBrand, _, _>(|n: usize| n.to_string()),
);
assert_eq!((exchange.get)("hello".to_string()), 5);
assert_eq!((exchange.set)(10), "10".to_string());Auto Trait Implementations
impl<'a, FunctionBrand, A, B, S, T> Freeze for Exchange<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, A>: Freeze,
<FunctionBrand as CloneableFn>::Of<'a, B, T>: Freeze,
impl<'a, FunctionBrand, A, B, S, T> RefUnwindSafe for Exchange<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, A>: RefUnwindSafe,
<FunctionBrand as CloneableFn>::Of<'a, B, T>: RefUnwindSafe,
impl<'a, FunctionBrand, A, B, S, T> Send for Exchange<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, A>: Send,
<FunctionBrand as CloneableFn>::Of<'a, B, T>: Send,
impl<'a, FunctionBrand, A, B, S, T> Sync for Exchange<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, A>: Sync,
<FunctionBrand as CloneableFn>::Of<'a, B, T>: Sync,
impl<'a, FunctionBrand, A, B, S, T> Unpin for Exchange<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, A>: Unpin,
<FunctionBrand as CloneableFn>::Of<'a, B, T>: Unpin,
impl<'a, FunctionBrand, A, B, S, T> UnsafeUnpin for Exchange<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, A>: UnsafeUnpin,
<FunctionBrand as CloneableFn>::Of<'a, B, T>: UnsafeUnpin,
impl<'a, FunctionBrand, A, B, S, T> UnwindSafe for Exchange<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, A>: UnwindSafe,
<FunctionBrand as CloneableFn>::Of<'a, B, T>: UnwindSafe,