pub struct Getter<'a, PointerBrand, S, T, A, B>where
PointerBrand: UnsizedCoercible,
S: 'a,
T: 'a,
A: 'a,
B: 'a,{
pub view_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, S, A>,
/* private fields */
}Expand description
A polymorphic getter.
Matches PureScript’s Getter s t a b.
Type Parameters
'a: The lifetime of the values.PointerBrand: The reference-counted pointer type.S: The source type of the structure.T: The target type of the structure.A: The source type of the focus.B: The target type of the focus.
Fields
view_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, S, A>Function to view the focus of the getter in a structure.
Implementations
Sourceimpl<'a, PointerBrand, S, T, A, B> Getter<'a, PointerBrand, S, T, A, B>where
PointerBrand: UnsizedCoercible,
S: 'a,
T: 'a,
A: 'a,
B: 'a,
Type Parameters
'a: The lifetime of the values.
PointerBrand: The reference-counted pointer type.
S: The source type of the structure.
T: The target type of the structure.
A: The source type of the focus.
B: The target type of the focus.
impl<'a, PointerBrand, S, T, A, B> Getter<'a, PointerBrand, S, T, A, B>where
PointerBrand: UnsizedCoercible,
S: 'a,
T: 'a,
A: 'a,
B: 'a,
Type Parameters
'a: The lifetime of the values.PointerBrand: The reference-counted pointer type.S: The source type of the structure.T: The target type of the structure.A: The source type of the focus.B: The target type of the focus.
Sourcepub fn new(view: impl 'a + Fn(S) -> A) -> Self
pub fn new(view: impl 'a + Fn(S) -> A) -> Self
Create a new getter from a view function.
Type Signature
forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (S -> A) -> Getter PointerBrand S T A B
Parameters
view: The view function.
Returns
A new instance of the type.
Examples
use fp_library::{
brands::RcBrand,
types::optics::Getter,
};
let g: Getter<RcBrand, (i32, String), (i32, String), i32, i32> = Getter::new(|(x, _)| x);
assert_eq!(g.view((42, "hi".to_string())), 42);Sourcepub fn view(&self, s: S) -> A
pub fn view(&self, s: S) -> A
View the focus of the getter in a structure.
Type Signature
forall PointerBrand S T A B. UnsizedCoercible PointerBrand => (&Getter PointerBrand S T A B, S) -> A
Parameters
&self: The getter instance.s: The structure to view.
Returns
The focus value.
Examples
use fp_library::{
brands::RcBrand,
types::optics::Getter,
};
let g: Getter<RcBrand, (i32, String), (i32, String), i32, i32> = Getter::new(|(x, _)| x);
assert_eq!(g.view((42, "hi".to_string())), 42);Trait Implementations
Sourceimpl<'a, PointerBrand, S, T, A, B> Clone for Getter<'a, PointerBrand, S, T, A, B>where
PointerBrand: UnsizedCoercible,
S: 'a,
T: 'a,
A: 'a,
B: 'a,
Type Parameters
'a: The lifetime of the values.
PointerBrand: The reference-counted pointer type.
S: The source type of the structure.
T: The target type of the structure.
A: The source type of the focus.
B: The target type of the focus.
impl<'a, PointerBrand, S, T, A, B> Clone for Getter<'a, PointerBrand, S, T, A, B>where
PointerBrand: UnsizedCoercible,
S: 'a,
T: 'a,
A: 'a,
B: 'a,
Type Parameters
'a: The lifetime of the values.PointerBrand: The reference-counted pointer type.S: The source type of the structure.T: The target type of the structure.A: The source type of the focus.B: The target type of the focus.
Sourcefn clone(&self) -> Self
fn clone(&self) -> Self
Type Signature
forall PointerBrand S T A B. UnsizedCoercible PointerBrand => &Getter PointerBrand S T A B -> Getter PointerBrand S T A B
Returns
A new Getter instance that is a copy of the original.
Examples
use fp_library::{
brands::RcBrand,
types::optics::Getter,
};
let g: Getter<RcBrand, (i32, String), (i32, String), i32, i32> = Getter::new(|(x, _)| x);
let cloned = g.clone();
assert_eq!(cloned.view((42, "hi".to_string())), 42);1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl<'a, PointerBrand, S, T, A, B, R, Q> Optic<'a, ForgetBrand<Q, R>, S, T, A, B> for Getter<'a, PointerBrand, S, T, A, B>where
PointerBrand: UnsizedCoercible,
Q: UnsizedCoercible + 'static,
S: 'a,
T: 'a,
A: 'a,
B: 'a,
R: 'a + 'static,
Type Parameters
'a: The lifetime of the values.
PointerBrand: The reference-counted pointer type for the getter.
S: The source type of the structure.
T: The target type of the structure.
A: The source type of the focus.
B: The target type of the focus.
R: The return type of the forget profunctor.
Q: The reference-counted pointer type for the forget brand.
impl<'a, PointerBrand, S, T, A, B, R, Q> Optic<'a, ForgetBrand<Q, R>, S, T, A, B> for Getter<'a, PointerBrand, S, T, A, B>where
PointerBrand: UnsizedCoercible,
Q: UnsizedCoercible + 'static,
S: 'a,
T: 'a,
A: 'a,
B: 'a,
R: 'a + 'static,
Type Parameters
'a: The lifetime of the values.PointerBrand: The reference-counted pointer type for the getter.S: The source type of the structure.T: The target type of the structure.A: The source type of the focus.B: The target type of the focus.R: The return type of the forget profunctor.Q: The reference-counted pointer type for the forget brand.
Sourcefn evaluate(
&self,
pab: <ForgetBrand<Q, R> as Kind_266801a817966495>::Of<'a, A, B>,
) -> <ForgetBrand<Q, R> as Kind_266801a817966495>::Of<'a, S, T>
fn evaluate( &self, pab: <ForgetBrand<Q, R> as Kind_266801a817966495>::Of<'a, A, B>, ) -> <ForgetBrand<Q, R> as Kind_266801a817966495>::Of<'a, S, T>
Type Signature
forall PointerBrand S T A B R Q. (UnsizedCoercible PointerBrand, UnsizedCoercible Q) => (&Getter PointerBrand S T A B R Q, Forget Q R A B) -> Forget Q R S T
Parameters
&self: The getter instance.pab: The profunctor value to transform.
Returns
The transformed profunctor value.
Examples
use fp_library::{
brands::{
optics::*,
*,
},
classes::optics::*,
functions::*,
types::optics::*,
};
let g: Getter<RcBrand, (i32, String), (i32, String), i32, i32> = Getter::new(|(x, _)| x);
let f = Forget::<RcBrand, i32, i32, i32>::new(|x| x);
let folded = Optic::<ForgetBrand<RcBrand, i32>, _, _, _, _>::evaluate(&g, f);
assert_eq!(folded.run((42, "hi".to_string())), 42);Auto Trait Implementations
impl<'a, PointerBrand, S, T, A, B> Freeze for Getter<'a, PointerBrand, S, T, A, B>
impl<'a, PointerBrand, S, T, A, B> RefUnwindSafe for Getter<'a, PointerBrand, S, T, A, B>where
<PointerBrand as RefCountedPointer>::CloneableOf<'a, dyn Fn(S) -> A + 'a>: RefUnwindSafe,
T: RefUnwindSafe,
B: RefUnwindSafe,
impl<'a, PointerBrand, S, T, A, B> Send for Getter<'a, PointerBrand, S, T, A, B>where
<PointerBrand as RefCountedPointer>::CloneableOf<'a, dyn Fn(S) -> A + 'a>: Send,
T: Sync,
B: Sync,
impl<'a, PointerBrand, S, T, A, B> Sync for Getter<'a, PointerBrand, S, T, A, B>where
<PointerBrand as RefCountedPointer>::CloneableOf<'a, dyn Fn(S) -> A + 'a>: Sync,
T: Sync,
B: Sync,
impl<'a, PointerBrand, S, T, A, B> Unpin for Getter<'a, PointerBrand, S, T, A, B>
impl<'a, PointerBrand, S, T, A, B> UnsafeUnpin for Getter<'a, PointerBrand, S, T, A, B>
impl<'a, PointerBrand, S, T, A, B> UnwindSafe for Getter<'a, PointerBrand, S, T, A, B>where
<PointerBrand as RefCountedPointer>::CloneableOf<'a, dyn Fn(S) -> A + 'a>: UnwindSafe,
T: RefUnwindSafe,
B: RefUnwindSafe,