pub struct Grating<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> {
pub run: <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>,
}Expand description
The Grating profunctor.
Type Parameters
'a: The lifetime of the functions.FunctionBrand: The cloneable function brand.A: The type of the value produced by the inner function.B: The type of the value consumed by the inner function.S: The source type of the structure.T: The target type of the structure.
Fields
run: <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>Grating function.
Implementations
Sourceimpl<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> Grating<'a, FunctionBrand, A, B, S, T>
Type Parameters
'a: The lifetime of the functions.
FunctionBrand: The cloneable function brand.
A: The type of the value produced by the inner function.
B: The type of the value consumed by the inner function.
S: The source type of the structure.
T: The target type of the structure.
impl<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> Grating<'a, FunctionBrand, A, B, S, T>
Type Parameters
'a: The lifetime of the functions.FunctionBrand: The cloneable function brand.A: The type of the value produced by the inner function.B: The type of the value consumed by the inner function.S: The source type of the structure.T: The target type of the structure.
Sourcepub fn new(
run: <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>,
) -> Self
pub fn new( run: <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>, ) -> Self
Creates a new Grating instance.
Type Signature
forall A B S T. (((S -> A) -> B) -> T) -> Grating FunctionBrand A B S T
Parameters
run: The grating function.
Returns
A new instance of the type.
Examples
use fp_library::{
brands::*,
functions::*,
types::optics::Grating,
};
let grating =
Grating::<RcFnBrand, i32, i32, (i32, i32), i32>::new(cloneable_fn_new::<RcFnBrand, _, _>(
|f: std::rc::Rc<dyn Fn(std::rc::Rc<dyn Fn((i32, i32)) -> i32>) -> i32>| {
let get_x = cloneable_fn_new::<RcFnBrand, _, _>(|(x, _)| x);
let get_y = cloneable_fn_new::<RcFnBrand, _, _>(|(_, y)| y);
f(get_x) + f(get_y)
},
));
let result = (grating.run)(cloneable_fn_new::<RcFnBrand, _, _>(
|g: std::rc::Rc<dyn Fn((i32, i32)) -> i32>| g((10, 20)),
));
assert_eq!(result, 30);Auto Trait Implementations
impl<'a, FunctionBrand, A, B, S, T> Freeze for Grating<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>: Freeze,
impl<'a, FunctionBrand, A, B, S, T> RefUnwindSafe for Grating<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>: RefUnwindSafe,
impl<'a, FunctionBrand, A, B, S, T> Send for Grating<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>: Send,
impl<'a, FunctionBrand, A, B, S, T> Sync for Grating<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>: Sync,
impl<'a, FunctionBrand, A, B, S, T> Unpin for Grating<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>: Unpin,
impl<'a, FunctionBrand, A, B, S, T> UnsafeUnpin for Grating<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>: UnsafeUnpin,
impl<'a, FunctionBrand, A, B, S, T> UnwindSafe for Grating<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, <FunctionBrand as CloneableFn>::Of<'a, S, A>, B>, T>: UnwindSafe,