IndexedSetter

Struct IndexedSetter 

pub struct IndexedSetter<'a, PointerBrand, I, S, T, A, B, F>
where F: IndexedSetterFunc<'a, I, S, T, A, B> + 'a,
{ pub setter_fn: F, /* private fields */ }
A polymorphic indexed setter.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • I: The index type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.
  • F: The setter function type.

Fields

setter_fn: F

The setter function.

Implementations

Source

impl<'a, PointerBrand, I, S, T, A, B, F> IndexedSetter<'a, PointerBrand, I, S, T, A, B, F>
where F: IndexedSetterFunc<'a, I, S, T, A, B> + 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • I: The index type.
  • S: The source type of the structure.
  • T: The target type of the structure after an update.
  • A: The source type of the focus.
  • B: The target type of the focus after an update.
  • F: The setter function type.
pub fn new(setter_fn: F) -> Self

Create a new indexed setter.

Type Signature

forall PointerBrand I S T A B F. IndexedSetterFunc F => F -> IndexedSetter PointerBrand I S T A B F

Parameters
  • setter_fn: The setter function.
Returns

A new IndexedSetter instance.

Examples
use fp_library::{
	brands::{
		RcBrand,
		optics::*,
	},
	types::optics::*,
};
struct MySetter;
impl<'a> IndexedSetterFunc<'a, usize, Vec<i32>, Vec<i32>, i32, i32> for MySetter {
	fn apply(
		&self,
		f: Box<dyn Fn(usize, i32) -> i32 + 'a>,
		s: Vec<i32>,
	) -> Vec<i32> {
		s.into_iter().enumerate().map(|(i, x)| f(i, x)).collect()
	}
}
let l: IndexedSetter<RcBrand, usize, Vec<i32>, Vec<i32>, i32, i32, MySetter> =
	IndexedSetter::new(MySetter);
assert_eq!(l.over(vec![10, 20], |i, x| x + (i as i32)), vec![10, 21]);
pub fn over(&self, s: S, f: impl Fn(I, A) -> B + 'a) -> T

Update the focus using an indexed function.

Type Signature

forall PointerBrand I S T A B F. IndexedSetterFunc F => (&IndexedSetter PointerBrand I S T A B F, S, (I, A) -> B) -> T

Parameters
  • &self: The indexed setter instance.
  • s: The structure to update.
  • f: The function to apply to the focus.
Returns

The updated structure.

Examples
use fp_library::{
	brands::{
		RcBrand,
		optics::*,
	},
	types::optics::*,
};
struct MySetter;
impl<'a> IndexedSetterFunc<'a, usize, Vec<i32>, Vec<i32>, i32, i32> for MySetter {
	fn apply(
		&self,
		f: Box<dyn Fn(usize, i32) -> i32 + 'a>,
		s: Vec<i32>,
	) -> Vec<i32> {
		s.into_iter().enumerate().map(|(i, x)| f(i, x)).collect()
	}
}
let l: IndexedSetter<RcBrand, usize, Vec<i32>, Vec<i32>, i32, i32, MySetter> =
	IndexedSetter::new(MySetter);
assert_eq!(l.over(vec![10, 20], |i, x| x + (i as i32)), vec![10, 21]);
pub fn set(&self, s: S, b: B) -> T
where B: Clone + 'a,

Set the focus.

Type Signature

forall PointerBrand I S T A B F. IndexedSetterFunc F => (&IndexedSetter PointerBrand I S T A B F, S, B) -> T

Parameters
  • &self: The indexed setter instance.
  • s: The structure to update.
  • b: The new focus value.
Returns

The updated structure.

Examples
use fp_library::{
	brands::{
		RcBrand,
		optics::*,
	},
	types::optics::*,
};
struct MySetter;
impl<'a> IndexedSetterFunc<'a, usize, Vec<i32>, Vec<i32>, i32, i32> for MySetter {
	fn apply(
		&self,
		f: Box<dyn Fn(usize, i32) -> i32 + 'a>,
		s: Vec<i32>,
	) -> Vec<i32> {
		s.into_iter().enumerate().map(|(i, x)| f(i, x)).collect()
	}
}
let l: IndexedSetter<RcBrand, usize, Vec<i32>, Vec<i32>, i32, i32, MySetter> =
	IndexedSetter::new(MySetter);
assert_eq!(l.set(vec![10, 20], 42), vec![42, 42]);
Source

impl<'a, PointerBrand, I, Brand, A, B> IndexedSetter<'a, PointerBrand, I, <Brand as Kind_cdc7cd43dac7585f>::Of<'a, A>, <Brand as Kind_cdc7cd43dac7585f>::Of<'a, B>, A, B, Mapped<Brand>>
where Brand: FunctorWithIndex<I>, A: 'a, B: 'a, I: 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The reference-counted pointer type.
  • I: The index type.
  • Brand: The brand of the functor.
  • A: The type of the elements in the structure.
  • B: The type of the elements in the result.
pub fn mapped() -> Self

Create an indexed setter from a FunctorWithIndex.

Type Signature

forall PointerBrand I Brand A B. FunctorWithIndex Brand => () -> IndexedSetter PointerBrand I Brand A B

Returns

A new IndexedSetter instance.

Examples
use fp_library::{
	brands::{
		RcBrand,
		VecBrand,
		optics::*,
	},
	functions::optics_indexed_set,
	types::optics::{
		IndexedSetter,
		Mapped,
	},
};
let l: IndexedSetter<RcBrand, usize, Vec<i32>, Vec<i32>, i32, i32, Mapped<VecBrand>> =
	IndexedSetter::mapped();
let v = vec![10, 20];
let s = optics_indexed_set::<RcBrand, _, _, _>(&l, v, 99);
assert_eq!(s, vec![99, 99]);