SetterPrime

Struct SetterPrime 

pub struct SetterPrime<'a, PointerBrand, S, A>
where PointerBrand: UnsizedCoercible, S: 'a, A: 'a,
{ pub over_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, (S, Box<dyn Fn(A) -> A + 'a>), S>, }
A concrete setter type where types do not change.

Matches PureScript’s Setter' s a.

Type Parameters

  • 'a: The lifetime of the values.
  • PointerBrand: The pointer brand for the function.
  • S: The type of the structure.
  • A: The type of the focus.

Fields

over_fn: <FnBrand<PointerBrand> as Kind_266801a817966495>::Of<'a, (S, Box<dyn Fn(A) -> A + 'a>), S>

Function to update the focus in a structure.

Implementations

Source

impl<'a, PointerBrand, S, A> SetterPrime<'a, PointerBrand, S, A>
where PointerBrand: UnsizedCoercible, S: 'a, A: 'a,

Type Parameters
  • 'a: The lifetime of the values.
  • PointerBrand: The pointer brand for the function.
  • S: The type of the structure.
  • A: The type of the focus.
pub fn new(over: impl 'a + Fn((S, Box<dyn Fn(A) -> A + 'a>)) -> S) -> Self

Create a new monomorphic setter.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => ((S, dyn (A -> A)) -> S) -> SetterPrime PointerBrand S A

Parameters
  • over: The over function.
Returns

A new instance of the type.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::SetterPrime,
};

let s: SetterPrime<RcBrand, (i32, String), i32> =
	SetterPrime::new(|(s, f): ((i32, String), Box<dyn Fn(i32) -> i32>)| (f(s.0), s.1));
assert_eq!(s.over((42, "hi".to_string()), |x| x + 1), (43, "hi".to_string()));
pub fn over(&self, s: S, f: impl Fn(A) -> A + 'a) -> S

Update the focus of the setter in a structure using a function.

Type Signature

forall PointerBrand S A. UnsizedCoercible PointerBrand => (&SetterPrime PointerBrand S A, S, A -> A) -> S

Parameters
  • &self: The setter instance.
  • s: The structure to update.
  • f: The function to apply to the focus.
Returns

The updated structure.

Examples
use fp_library::{
	brands::RcBrand,
	types::optics::SetterPrime,
};

let s: SetterPrime<RcBrand, (i32, String), i32> =
	SetterPrime::new(|(s, f): ((i32, String), Box<dyn Fn(i32) -> i32>)| (f(s.0), s.1));
assert_eq!(s.over((42, "hi".to_string()), |x| x + 1), (43, "hi".to_string()));