pub struct Stall<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> {
pub get: <FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>,
pub set: <FunctionBrand as CloneableFn>::Of<'a, (S, B), T>,
}Expand description
The Stall profunctor.
Type Parameters
'a: The lifetime of the functions.FunctionBrand: The cloneable function brand.A: The type of the value produced by the preview function.B: The type of the value consumed by the setter.S: The source type of the structure.T: The target type of the structure.
Fields
get: <FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>Preview function: tries to extract the focus.
set: <FunctionBrand as CloneableFn>::Of<'a, (S, B), T>Setter function.
Implementations
Sourceimpl<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> Stall<'a, FunctionBrand, A, B, S, T>
Type Parameters
'a: The lifetime of the functions.
FunctionBrand: The cloneable function brand.
A: The type of the value produced by the preview function.
B: The type of the value consumed by the setter.
S: The source type of the structure.
T: The target type of the structure.
impl<'a, FunctionBrand: CloneableFn, A: 'a, B: 'a, S: 'a, T: 'a> Stall<'a, FunctionBrand, A, B, S, T>
Type Parameters
'a: The lifetime of the functions.FunctionBrand: The cloneable function brand.A: The type of the value produced by the preview function.B: The type of the value consumed by the setter.S: The source type of the structure.T: The target type of the structure.
Sourcepub fn new(
get: <FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>,
set: <FunctionBrand as CloneableFn>::Of<'a, (S, B), T>,
) -> Self
pub fn new( get: <FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>, set: <FunctionBrand as CloneableFn>::Of<'a, (S, B), T>, ) -> Self
Creates a new Stall instance.
Type Signature
forall A B S T. (S -> Result A T, (S, B) -> T) -> Stall FunctionBrand A B S T
Parameters
get: The preview function.set: The setter function.
Returns
A new instance of the type.
Examples
use fp_library::{
brands::RcFnBrand,
classes::cloneable_fn::new as cloneable_fn_new,
types::optics::Stall,
};
let stall = Stall::<RcFnBrand, i32, i32, (i32, i32), (i32, i32)>::new(
cloneable_fn_new::<RcFnBrand, _, _>(|s: (i32, i32)| Ok(s.0)),
cloneable_fn_new::<RcFnBrand, _, _>(|(s, b): ((i32, i32), i32)| (b, s.1)),
);
assert_eq!((stall.get)((10, 20)), Ok(10));
assert_eq!((stall.set)(((10, 20), 30)), (30, 20));Auto Trait Implementations
impl<'a, FunctionBrand, A, B, S, T> Freeze for Stall<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>: Freeze,
<FunctionBrand as CloneableFn>::Of<'a, (S, B), T>: Freeze,
impl<'a, FunctionBrand, A, B, S, T> RefUnwindSafe for Stall<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>: RefUnwindSafe,
<FunctionBrand as CloneableFn>::Of<'a, (S, B), T>: RefUnwindSafe,
impl<'a, FunctionBrand, A, B, S, T> Send for Stall<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>: Send,
<FunctionBrand as CloneableFn>::Of<'a, (S, B), T>: Send,
impl<'a, FunctionBrand, A, B, S, T> Sync for Stall<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>: Sync,
<FunctionBrand as CloneableFn>::Of<'a, (S, B), T>: Sync,
impl<'a, FunctionBrand, A, B, S, T> Unpin for Stall<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>: Unpin,
<FunctionBrand as CloneableFn>::Of<'a, (S, B), T>: Unpin,
impl<'a, FunctionBrand, A, B, S, T> UnsafeUnpin for Stall<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>: UnsafeUnpin,
<FunctionBrand as CloneableFn>::Of<'a, (S, B), T>: UnsafeUnpin,
impl<'a, FunctionBrand, A, B, S, T> UnwindSafe for Stall<'a, FunctionBrand, A, B, S, T>where
<FunctionBrand as CloneableFn>::Of<'a, S, Result<A, T>>: UnwindSafe,
<FunctionBrand as CloneableFn>::Of<'a, (S, B), T>: UnwindSafe,