pub struct Pair<First, Second>(pub First, pub Second);Expand description
Wraps two values.
A simple tuple struct that holds two values of potentially different types.
Higher-Kinded Type Representation
This type has multiple higher-kinded representations:
PairBrand: fully polymorphic over both values (bifunctor).PairWithFirstBrand<First>: the first value type is fixed, polymorphic over the second (functor over second).PairWithSecondBrand<Second>: the second value type is fixed, polymorphic over the first (functor over first).
Serialization
This type supports serialization and deserialization via serde when the serde feature is enabled.
First: The type of the first value.Second: The type of the second value.
Fields
0: The first value.1: The second value.
Examples
use fp_library::types::*;
let p = Pair(1, "hello");
assert_eq!(p.0, 1);
assert_eq!(p.1, "hello");Tuple Fields
0: First1: SecondTrait Implementations
Sourceimpl<'de, First, Second> Deserialize<'de> for Pair<First, Second>where
First: Deserialize<'de>,
Second: Deserialize<'de>,
impl<'de, First, Second> Deserialize<'de> for Pair<First, Second>where
First: Deserialize<'de>,
Second: Deserialize<'de>,
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl<First: Ord, Second: Ord> Ord for Pair<First, Second>
impl<First: Ord, Second: Ord> Ord for Pair<First, Second>
1.21.0 · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl<First: PartialOrd, Second: PartialOrd> PartialOrd for Pair<First, Second>
impl<First: PartialOrd, Second: PartialOrd> PartialOrd for Pair<First, Second>
impl<First: Copy, Second: Copy> Copy for Pair<First, Second>
impl<First: Eq, Second: Eq> Eq for Pair<First, Second>
impl<First, Second> StructuralPartialEq for Pair<First, Second>