SendEndofunction

Struct SendEndofunction 

pub struct SendEndofunction<'a, FnBrand: SendLiftFn, A: 'a>(pub <FnBrand as SendCloneFn>::Of<'a, A, A>);
A thread-safe wrapper for endofunctions that enables monoidal operations.

SendEndofunction a represents a function a -> a wrapped in an Arc<dyn Fn>.

It exists to provide a monoid instance where:

  • The binary operation append is function composition.
  • The identity element empty is the identity function.

This is the Send + Sync counterpart of Endofunction.

Type Parameters

  • 'a: The lifetime of the function and its captured data.
  • FnBrand: The brand of the thread-safe cloneable function wrapper.
  • A: The input and output type of the function.

Tuple Fields

0: <FnBrand as SendCloneFn>::Of<'a, A, A>

The wrapped function.

Implementations

Source

impl<'a, FnBrand: SendLiftFn, A: 'a> SendEndofunction<'a, FnBrand, A>

Type Parameters
  • 'a: The lifetime of the function and its captured data.
  • FnBrand: The brand of the function (e.g., ArcFnBrand).
  • A: The input and output type of the function.
pub fn new(f: <FnBrand as SendCloneFn>::Of<'a, A, A>) -> Self

Creates a new SendEndofunction.

Type Signature

forall A. SendLiftFn FnBrand => FnBrand A A -> SendEndofunction FnBrand A

Parameters
  • f: The function to wrap.
Returns

A new SendEndofunction.

Examples
use fp_library::{
	brands::*,
	functions::*,
	types::*,
};

let f =
	SendEndofunction::<ArcFnBrand, _>::new(send_lift_fn_new::<ArcFnBrand, _, _>(|x: i32| {
		x * 2
	}));
assert_eq!(f.0(5), 10);